
# q-expansion of newform 8550.2.a.e, downloaded from the LMFDB on 26 April 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 8550
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[-1], [0], [0], [-2], [0], [-2], [0], [1], [0], [6], [2], [-2], [0], [-8], [0], [6], [6], [2], [4], [0], [-14], [2], [6], [12], [10], [6], [-8], [0], [-16], [-6], [16], [0], [-12], [-4], [6], [-10], [4], [4], [0], [-18], [-6], [-16], [0], [-2], [18], [-4], [8], [4], [12], [-10], [24], [-24], [-22], [-12], [6], [-24], [-18], [20], [28], [-24], [-32], [-6], [-20], [0], [10], [-30], [20], [-2], [18], [-10], [12], [-24], [10], [-14], [8], [-24], [30], [-8], [-12], [-22], [-12], [8], [24], [-2], [-22], [-30], [-24], [22], [-30], [-26], [6], [0], [16], [0], [20], [-12], [6], [-12], [4], [-10], [4], [6], [-24], [-24], [20], [-2], [18], [36], [-12], [26], [-32], [16], [12], [20], [32], [24], [-32], [12], [-18], [-6], [32], [-14], [-30], [-12], [-28], [-30], [-10], [48], [34], [-32], [44], [-48], [-22], [-32], [-6], [-34], [-18], [-44], [42], [-54], [-16], [-42], [22], [48], [8], [24], [-44], [-18], [-4], [0], [-38], [30], [16], [-48], [-44], [-60], [-40], [6], [-26], [-6], [-18], [30], [-26], [42], [-42], [24], [-58], [-44], [2], [-18], [-6], [-16], [48], [-14], [62], [-6], [-4], [-6], [28], [-10], [46], [30], [-26], [30], [-24], [30], [-38], [-20], [50], [24], [34], [54], [8], [-6], [54], [-6], [-58], [4], [12], [-48], [-30], [-10], [-26], [14], [0], [-30], [20], [24], [54], [-64], [58], [42], [10], [-12], [12], [-34], [22], [-12], [0], [30], [-4], [26], [0], [-26], [-12], [-34], [30], [-24], [-8], [-24], [28], [44], [-22], [-60], [28], [0], [-34], [54], [12], [-72], [36], [-52], [10], [20], [42], [24], [-38], [18], [56], [48], [-32], [54], [12], [62], [-6], [-60], [26], [76], [42], [10], [-44], [72], [-46], [-14], [24], [68], [-30], [-18], [-44], [54], [8], [52], [10], [-4], [46], [10], [66], [20], [50], [-72], [-48], [-40], [12], [-64], [4], [24], [22], [6], [32], [-36], [18], [30], [18], [42], [-14], [-30], [26], [-30], [66], [40], [58], [-18], [56], [24], [-4], [-38], [-48], [-28], [24], [-8], [-36], [66], [-36], [4], [72], [-34], [84], [36], [58], [18], [-40], [10], [6], [-8], [48], [74], [80], [76], [-72], [-66], [26], [30], [-64], [-18], [-52], [54], [44], [-66], [50], [70], [10], [-48], [30], [62], [-78], [78], [-70], [-44], [-48], [42], [20], [-26], [-18], [-80], [-40], [-66], [72], [72], [-60], [-24], [28], [90], [72], [-30], [-56], [-62], [18], [64], [62], [-48], [-4], [36], [54], [44], [-20], [-18], [-48], [70], [-18], [-14], [-30], [72], [-38], [-72], [-16], [24], [68], [76], [-20], [-72], [86], [-78], [-84], [-92], [-60], [22], [14], [-60], [-40], [54], [56], [-48], [4], [78], [78], [68], [28], [-72], [4], [48], [-14], [-66], [-36], [44], [-86], [-18], [-84], [28], [12], [-24], [-6], [-38], [-24], [-30], [10], [-78], [-96], [30], [-4], [24], [-22], [-72], [-76], [0], [-68], [66], [86], [92], [-44], [-52], [6], [-42], [-16], [84], [98], [-30], [76], [-48], [-10], [-100], [28], [0], [54], [-66], [-62], [-6], [68], [66], [64], [-90], [56], [50], [6], [44], [40], [10], [-46], [6], [-66], [-100], [28], [36], [36], [-34], [18], [-86], [6], [92], [48], [-42], [58], [60], [-2], [-42], [46], [18], [50], [78], [-64], [-22], [-98], [96], [-94], [18], [0], [-10], [4], [-78], [-40], [-88], [18], [-2], [102]]
